import json
import requests
import logging
import os
import sys
import traceback
# import urllib
from gempyp.config import DefaultSettings


# upload_file_api = DefaultSettings.getUrls('bucket-file-upload-api')
# upload_data_api = DefaultSettings.getUrls('bucket-data-upload-api')
# delete_file = DefaultSettings.getUrls('bucket-file-modify')

def upload_to_s3(api=None, tag=None, file=None, data=None, s_run_id=None, folder=None, username=None, bearer_token=None, bridge_token=None):
    logging.info("In Upload section")
    if bearer_token is not None:
        headers = {"Authorization": "Bearer {}".format(bearer_token)}
    elif bridge_token is not None:
        if username is None:
            logging.ERROR("------ Please pass username along with Bridge token -----")
            sys.exit()
        headers = {"bridgeToken": bridge_token, "username": username}
    params = dict()
    if folder is not None:
        params["folder"] = folder
    if tag is not None: 
        params["tag"] = tag
    if s_run_id is not None:
        params["s_run_id"] = s_run_id
    if data is not None:
        logging.info("------ Uploading data to s3 -----")
        try:
            api = DefaultSettings.getUrls('bucket-data-upload-api') if not api else api
            headers["Content-Type"] = "text/plain"
            params["file"] = file
            try:
                data = str(data).encode('utf-8')
            except Exception as e:
                logging.warn(e)
                traceback.print_exc()
            response = requests.post(api, params=params, data=data, headers=headers)
            data = json.loads(response.text)  
            if response.status_code == 200:
                data = json.loads(response.text)    
                data_info = data["data"]
                return data_info
        except Exception as e:
            traceback.print_exc()
            logging.warn(e)
    elif file is not None:
        file = file.split(",")
        files = list()
        logging.info("---- Uploading file to s3 ----")
        api = DefaultSettings.getUrls('bucket-file-upload-api') if not api else api
        for f in file:
            if not os.path.isfile(f) or f == "N.A":
                logging.error("Path of file invalid - " + str(f))
                continue
            files.append(("file", open(f, "rb")))
        response = requests.post(api, files=files, params=params, headers=headers)  
        data = json.loads(response.text) 
        if response.status_code == 200:
            data = json.loads(response.text)    
            file_info = data["data"]

            return file_info

def download_from_s3(api, bearer_token=None, bridge_token=None, username=None, id=None):
    logging.info("In Download section")
    if bearer_token is not None:
        headers = {"Authorization": "Bearer {}".format(bearer_token)}
    elif bridge_token is not None:
        if username is None:
            logging.ERROR("------ Please pass username along with Bridge token -----")
            sys.exit()
        headers = {"bridgeToken": bridge_token, "username": username}
    params = dict()
    if '?' not in api and id is None:
        logging.error("Please pass download url or the id")
        sys.exit()
    elif '?' in api:
        api_query = api.split('?')
        params = api_query[1][3:]
        api = api_query[0]
        params = {"id": "{}".format(params)}
    elif '?' not in api and id is not None:
        params = {"id": id}
    response = requests.get(api, params=params, headers=headers)  
    return response.text


def create_s3_link(**kwargs):
    """ creating s3 link to be viewed on file viewer on jewel UI"""

    s3_viewer = DefaultSettings.getUrls('file-viewer')
    if kwargs.get("url", None):
        params = "url=" + kwargs.get("url")
        kwargs.pop("url")
        for key, value in kwargs.items():
            params = f"{params}&{key}={value}"
        return f"{s3_viewer}?{params}"
    return None

if __name__ == "__main__":
    print(download_from_s3(api="https://apis-beta.gemecosystem.com/v1/download/file?id=gem-np:PYPRESTBEFOREFILE:BeforeAfterFile1.py",username="tanya.agarwal",bridge_token="374efe42-323e-4445-b89b-1ff750f000c61664541163883"))


from abc import ABC, abstractmethod
from typing import Dict
class abstaractExecutor(ABC):

    @abstractmethod
    def execute(self, testcaseSettings: Dict) -> Dict:
        """
            the method which will run the single testcase file and parses the results.
            this method can be run in parallel
            its implemetations can run somewhere else getting the settings from a message queue
        """
        pass


import traceback
import os
import uuid
import getpass
from gempyp.libs.gem_s3_common import upload_to_s3, create_s3_link
from gempyp.config import DefaultSettings


def writeToReport(dv_obj):
    """
    This function takes DVM obj,
    creates the testcase report if the runmode is debug mode,
    Creates the dictionary that is to be sent to gempyp"""
    result = {}
    if not dv_obj.reporter.result_file_name:
        try:
            try:
                dv_obj.reporter.finalizeReport()   ## need to test
                if dv_obj.data.get("OUTPUT_FOLDER", dv_obj.default_report_path) is None:
                    os.makedirs(dv_obj.data.get("OUTPUT_FOLDER", dv_obj.default_report_path))
            except Exception as e:
                dv_obj.logger.info(traceback.print_exc())
            dv_obj.reporter.json_data = dv_obj.reporter.template_data.makeTestcaseReport()
            dv_obj.json_data = dv_obj.reporter.json_data
            result = dv_obj.reporter.serialize()
        except Exception as e:
            # dv_obj.logger.info(traceback.print_exc())
            traceback.print_exc()
    result = dv_obj.reporter.serialize()
    output = []
    tempdict = {} 
    unique_id = uuid.uuid4()
    try:
        if os.environ.get('unique_id'):
            unique_id = os.environ.get('unique_id')
            # unique_id = unique_id.split(dv_obj.data.get("ENV").upper()).strip("_")
    except Exception:
        traceback.print_exc()
    tc_run_id = f"{dv_obj.tcname}_{unique_id}"
    tempdict["tc_run_id"] = tc_run_id
    tempdict["name"] = result["NAME"]
    tempdict["category"] = dv_obj.category
    tempdict["status"] = result["STATUS"]
    tempdict["base_user"] = getpass.getuser()
    tempdict["invoke_user"] = dv_obj.data.get("INVOKE_USER")
    tempdict["machine"] = dv_obj.data.get("MACHINE")
    tempdict["product_type"] = "GEMPYP-DV"
    tempdict["steps"] = result["json_data"]['steps']
    tempdict["result_file"] = result["RESULT_FILE"]
    tempdict["start_time"] = result["START_TIME"]
    tempdict["end_time"] = result["END_TIME"]
    tempdict["ignore"] = False
    all_status = result["json_data"]["meta_data"][2]
    total = 0
    for key in all_status:
        total += all_status[key]
    result["json_data"]["meta_data"][2]["TOTAL"] = total

    # getting the log file ( the custom gempyp logger)
    try:
        s3_log_file_url = create_s3_link(url=upload_to_s3(DefaultSettings.urls["data"]["bucket-file-upload-api"], bridge_token=dv_obj.data.get("SUITE_VARS", None).get("bridge_token",None), username=dv_obj.data.get("SUITE_VARS", None).get("username",None), file=dv_obj.configData.get("log_path", dv_obj.configData.get("LOG_PATH","N.A")),tag="public")[0]["Url"])
        s3_log_file_url = f'<a href="{s3_log_file_url}" target=_blank>view</a>'
    except Exception:
        s3_log_file_url=None
    tempdict["log_file"] = dv_obj.configData.get("log_path","N.A")

    singleTestcase = {}
    singleTestcase["testcase_dict"] = tempdict
    singleTestcase["misc"] = result.get("MISC")
    singleTestcase["json_data"] = dv_obj.json_data
    singleTestcase["misc"]["log_file"] = s3_log_file_url
    output.append(singleTestcase)
    return output



import uuid
from gempyp.libs.gem_s3_common import upload_to_s3, create_s3_link
from gempyp.config import DefaultSettings
import mysql.connector
import os
import uuid
from configparser import ConfigParser
from typing import Dict
import pandas as pd
from gempyp.engine.baseTemplate import TestcaseReporter as Base
from gempyp.libs.common import moduleImports
from gempyp.libs.enums.status import status
from gempyp.libs.common import readPath
from gempyp.dv.dvReporting import writeToReport
# from telnetlib import STATUS
import traceback
import pandas as pd
import logging
import math
import numpy
import pg8000
from gempyp.dv.dvObj import DvObj
from gempyp.libs.common import download_common_file
from gempyp.engine.runner import getError
from gempyp.libs import common

class DvRunner(Base):

    def __init__(self, data):
        
        self.data = data
        self.configData: Dict = self.data.get("config_data")
        self.logger = data["config_data"]["LOGGER"] if "LOGGER" in data["config_data"].keys() else logging
        self.logger.info("---------------------Inside DV FRAMEWORK------------------------")
        self.logger.info(f"-------Executing testcase - \"{self.data['config_data']['NAME']}\"---------")
        if data.get("default_urls", None):
            DefaultSettings.urls.update(data.get("default_urls"))   # only for optimized mode, urls not shared between processes
        # # set vars
        self.setVars()
        # # setting self.reporter object
        self.logger.info("--------------------Report object created ------------------------")
        self.reporter = Base(project_name=self.project, testcase_name=self.tcname)

    def dvEngine(self):
        try:
            self.validate()
        except Exception as e:
            self.logger.error(traceback.print_exc())
            self.reporter.addRow("Executing Test steps", f'Something went wrong while executing the testcase- {str(e)}', status.ERR)
            common.errorHandler(self.logger, e, "Error occured while running the testcase")
            error_dict = getError(e, self.data["config_data"])
            error_dict["json_data"] = self.reporter.serialize()
            return None, error_dict

        try:
            column = []
            try:
                if "DATABASE" in self.configData:
                    if self.configData["DATABASE"].lower() == 'custom': 
                        pass
                    else:
                        configPath= self.configData["DB_CONFIG_PATH"]
                        configur = ConfigParser()
                        config = readPath(configPath)
                        configur.read(config)
            except Exception as e:
                self.reporter.addRow("Config File","Path is not Correct",status.FAIL)
                traceback.print_exc()
                output = writeToReport(self)
                return output, None
            try:
                    # in custom mode we are sending host name using source cred while in other mode we are sending credentenials using sourceCred
                if 'DATABASE' in self.configData:    
                    if self.configData["DATABASE"].lower() == 'custom': 
                        if 'SOURCE_CONN' in self.configData:
                            sourceCred = self.getHost('SOURCE_CONN')
                        if 'TARGET_CONN' in self.configData:
                            targetCred = self.getHost('TARGET_CONN')

                    else:
                        self.logger.info("----Parsing the Config File----")
                        if 'SOURCE_DB' in self.configData:
                            sourceCred ={"host":configur.get(self.configData["SOURCE_DB"],'host'),"dbName":configur.get(self.configData["SOURCE_DB"],'databaseName'),"userName":configur.get(self.configData["SOURCE_DB"],'userName'),"password":configur.get(self.configData["SOURCE_DB"],'password'),"port":configur.get(self.configData["SOURCE_DB"],'port')}
                        if 'TARGET_DB' in self.configData:
                            targetCred ={"host":configur.get(self.configData["TARGET_DB"],'host'),"dbName":configur.get(self.configData["TARGET_DB"],'databaseName'),"userName":configur.get(self.configData["TARGET_DB"],'userName'),"password":configur.get(self.configData["TARGET_DB"],'password'),"port":configur.get(self.configData["TARGET_DB"],'port')} 
                        self.reporter.addRow("Parsing DB Conf","Parsing of DB config is Successfull",status.PASS)
            except Exception as e:
                self.logger.error(str(e))
                traceback.print_exc()
                self.reporter.addRow("Parsing DB Conf","Exception Occurred",status.FAIL)
                output = writeToReport(self)
                self.addReasonOfFailure(traceback)
                return output, None
            self.keys = self.configData["KEYS"].split(',')
            self.reporter.addMisc("KEYS",", ".join(self.keys))
            # parsing row and connection row
            self.li1 = []
            self.li2 = []
            for i in self.keys:
                column.append(i)
                self.li1.append([])
                self.li2.append([])
            """checking whether data is present in csv form or in db"""
            if 'SOURCE_CSV' in self.configData:
                try:
                    self.logger.info("Getting Source_CSV File Path")
                    path = self.configData['SOURCE_CSV']
                    sourceCsvPath = readPath(path)
                    file_obj = download_common_file(sourceCsvPath,self.data.get("SUITE_VARS",None))
                    sourceDelimiter = self.configData.get('SOURCE_DELIMITER',',')
                    self.source_df, self.source_columns = self.csvFileReader(file_obj, sourceDelimiter, "source")
                except Exception as e:
                    self.logger.error(str(e))
                    traceback.print_exc()
                    self.reporter.addRow("Parsing Source File Path","Exception Occurred",status.FAIL)
                    output = writeToReport(self)
                    self.addReasonOfFailure(traceback)
                    return output, None
            else:
                """Connecting to sourceDB"""
                self.source_df, self.source_columns = self.connectDB(sourceCred, "SOURCE")

            if 'TARGET_CSV' in self.configData:
                try:
                    self.logger.info("Getting Target_CSV File Path")
                    path = self.configData['TARGET_CSV']
                    targetCsvPath = readPath(path)
                    file_obj = download_common_file(targetCsvPath,self.data.get("SUITE_VARS",None))
                    targetDelimiter = self.configData.get('TARGET_DELIMITER',',')
                    self.target_df, self.target_columns = self.csvFileReader(file_obj, targetDelimiter, "Target")
                    # self.target_columns = self.target_df.columns.values.tolist()
                except Exception as e:
                    self.logger.error(str(e))
                    traceback.print_exc()
                    self.reporter.addRow("Parsing Target File Path","Exception Occurred",status.FAIL)
                    output = writeToReport(self)
                    self.addReasonOfFailure(traceback)
                    return output, None
            else:
                """Connecting to TargetDB"""
                self.target_df, self.target_columns = self.connectDB(targetCred, "TARGET")
            if "BEFORE_FILE" in self.configData:
                self.beforeMethod()
                self.li1 = []
                self.li2 = []
                for i in self.keys:
                    column.append(i)
                    self.li1.append([])
                    self.li2.append([])
                self.matchKeys(self.source_columns,"SOURCE")
                self.matchKeys(self.target_columns,"TARGET")
            try:
                if self.source_columns==self.target_columns:
                    pass
                else:
                    raise Exception
            except Exception:
                self.reporter.addRow("Column in Table","Not Found",status.FAIL)
                self.logger.info("--------Same Column not Present in Both Table--------")
                output = writeToReport(self)
                self.addReasonOfFailure(traceback)
                return output, None
            
            self.df_compare(self.source_df, self.target_df, self.keys)
            self.reporter.finalizeReport()
            output = writeToReport(self)
            return output, None
        except Exception as e:
            self.logger.error(str(e))
            traceback.print_exc()
            self.addReasonOfFailure(traceback)
            output = writeToReport(self)
            return output, None

    def csvFileReader(self, path, delimiter, db):
        try:
            self.logger.info(f"Reading data from {db} CSV File")
            df = pd.read_csv(path, delimiter)
            df = df.loc[:, ~df.columns.str.contains('^Unnamed')]
            columns = df.columns.values.tolist()
            self.reporter.addRow(f"Parsing {db} File","Parsing File is Successfull",status.PASS)
            self.matchKeys(columns, db)
            return df, columns
        except Exception:
            raise Exception

    def validate(self):
        if "KEYS" in self.configData:
            if 'SOURCE_CONN' in self.configData or 'SOURCE_CSV' in self.configData or 'SOURCE_DB' in self.configData:
                pass
            else:
                raise Exception("Tags for Source connection not Present in Config, Please review config.xml")
        else:
            raise Exception("Keys not Present in Config, Please recheck")

    def connectDB(self,cred, db):
        try:
            log = f"----Connecting to {db}DB----"
            self.logger.info(log)
            dbType = f"{db}DB"
            db_conn = f"{db}_CONN"
            myDB = self.connectingDB(dbType, db_conn, cred)
            myCursor = myDB.cursor()
        except Exception as e:
            self.logger.error(str(e))
            self.reporter.addRow(f"Connection to {db}DB: "+ str(cred["host"]),"Exception Occurred",status.FAIL)
            self.addReasonOfFailure(traceback)
            raise e
                
        try:
            self.logger.info(f"----Executing the {db}SQL----")
            sql = f"{db}_SQL"
            myCursor.execute(self.configData[sql])
            self.reporter.addRow(f"Executing {db} SQL",f"{db} SQL executed Successfull",status.PASS)
            columns = [i[0] for i in myCursor.description]
        except Exception as e:
            self.logger.error(str(e))
            self.reporter.addRow(f"Executing {db} SQL","Exception Occurred",status.FAIL)
            output = writeToReport(self)
            self.addReasonOfFailure(traceback)
            raise e
        
        self.matchKeys(columns, db)
        results = myCursor.fetchall()
        db_1 = pd.DataFrame(results, 
                                columns=columns)
        myDB.close()
        return db_1, columns

    def getHost(self, conn):
        conn_string = self.configData[conn]
        ind = conn_string.index('host')
        count = 0
        li = []
        for i in range(ind,len(conn_string)):
            if conn_string[i] == "'":
                count +=1
                li.append(i)
                if count == 2:
                    break
        host = conn_string[li[0]+1:li[1]]
        return {'host': host}

    def connectingDB(self,db, conn, cred ):

            if self.configData["DATABASE"].lower() == 'custom':
                db1 = self.configData[conn]
                ind = db1.index("(")
                module = db1[0:ind-8:1]
                __import__(module)
                myDB = eval(db1)
                self.reporter.addRow(f"Connection to {db}: "+ cred['host'],f"Connection to {db} is Successfull",status.PASS)
            elif self.configData["DATABASE"].lower() == 'mysql':
                myDB = mysql.connector.connect(host= cred['host'],user = cred["userName"], password = cred['password'],database= cred['dbName'], port = cred['port'])
                self.reporter.addRow(f"Connection to {db}: "+ str(cred["host"]),f"Connection to {db} is Successfull",status.PASS)
            elif self.configData["DATABASE"].lower() == 'postgresql':
                myDB = pg8000.connect(host= cred['host'],user = cred["userName"], password = cred['password'],database= cred['dbName'], port = cred['port'])
                self.reporter.addRow(f"Connection to {db}: "+ str(cred["host"]),f"Connection to {db} is Successfull",status.PASS)
            return myDB     


    def matchKeys(self, columns, db):

        try:
            self.logger.info(f"Matching Keys in {db} DB")
            key = []
            for i in self.keys:
                if i not in columns:
                    key.append(i)
            if len(key)==0:
                self.reporter.addRow(f"Matching Given Keys in {db}",f"Keys are Present in {db}",status.PASS)
                self.logger.info(f"Given Keys are Present in {db} DB")
            else:
                raise (f"Keys are not Present in {db}")   
        except Exception as e:
            keyString1 = ", ".join(key)
            self.reporter.addRow(f"Matching Given Keys in {db}","Keys: " + keyString1 +f" are not Present in {db}DB",status.FAIL)
            self.logger.info("------Given Keys are not present in DB------")


    def df_compare(self,df_1, df_2, key):
    
        try:
            self.logger.info("In df_compare Function")
            self.df_1 = df_1
            self.df_2 = df_2
            self.df_1['key'] = self.df_1[key].apply(lambda row: '--'.join(row.values.astype(str)), axis=1)
            self.df_2['key'] = self.df_2[key].apply(lambda row: '--'.join(row.values.astype(str)), axis=1)
            self.df_1.set_index('key', inplace=True)
            self.df_2.set_index('key', inplace=True)
            self.df_1.drop(key, axis=1, inplace=True)
            self.df_2.drop(key, axis=1, inplace=True)
            src_key_values = self.df_1.index.values
            tgt_key_values = self.df_2.index.values
            self.headers = list(set(list(df_1.columns)) - set(key+['key']))
            self.common_keys = list(set(src_key_values) & set(tgt_key_values))
            self.keys_only_in_src = list(set(src_key_values) - set(tgt_key_values))
            self.keys_only_in_tgt = list(set(tgt_key_values) - set(src_key_values))
            
            self.value_check = 0
            self.key_check = 0
            self.dict1 = { 'REASON OF FAILURE':[]}
            self.key_dict = {}
            value_dict = self.addCommonExcel(self.common_keys)
            """calling src and tgt for getting different keys"""
            srcDict = self.addExcel(self.keys_only_in_src, "Source")
            tgtDict = self.addExcel(self.keys_only_in_tgt, "Target")
            self.key_check = len(self.key_dict["REASON OF FAILURE"])
            self.value_check = len(value_dict["REASON OF FAILURE"])
            self.writeExcel(value_dict,self.key_dict)
        except Exception:
            traceback.print_exc()
            self.addReasonOfFailure(traceback)


    def setVars(self):

        """
        For setting variables like testcase name, output folder etc.
        """
        self.default_report_path = os.path.join(os.getcwd(), "pyprest_reports")
        self.data["OUTPUT_FOLDER"] = self.data.get("OUTPUT_FOLDER", self.default_report_path)
        if self.data["OUTPUT_FOLDER"].strip(" ") == "":
            self.data["OUTPUT_FOLDER"] = self.default_report_path
        self.project = self.data["PROJECT_NAME"]
        self.tcname = self.data["config_data"]["NAME"]
        self.env = self.data["ENV"]
        self.category = self.data["config_data"].get("CATEGORY", None)
    

    def truncate(self,f, n):

        return math.floor(f * 10 ** n) / 10 ** n
    
    
    def addExcel(self, _list, db):

            if _list:
                for i in _list:
                    li = i.split("--")
                    for i in range(len(li)):
                        key = self.keys[i]
                        self.li1[i].append(li[i])
                        value = self.li1[i]
                        self.key_dict[key] = value
                    self.dict1.get("REASON OF FAILURE").append(f"keys only in {db}")
            self.key_dict.update(self.dict1)
            return self.key_dict


    def addCommonExcel(self,commonList:list):  

            self.logger.info("In addCommonExcel Function")
            dummy_dict = { 'Column_Name':[],'Source_Value':[],'Target_Value':[],'REASON OF FAILURE':[]}
            comm_dict ={}

            if commonList:
                for key_val in commonList:
                    for field in self.headers:
                        src_val = self.df_1.loc[key_val,field]
                        tgt_val = self.df_2.loc[key_val,field]
                        # if type(src_val) == pd.core.series.Series:
                        #     val = src_val[len(src_val)-1]
                        #     src_val = val
                        # if type(tgt_val) == pd.core.series.Series:
                        #     val = tgt_val[len(tgt_val)-1]
                        #     tgt_val = val
                        if src_val == src_val or tgt_val == tgt_val:
                            if "THRESHOLD" in self.configData:
                                self.reporter.addMisc("Threshold",str(self.configData["THRESHOLD"]))
                                if type(src_val)== numpy.float64 and math.isnan(src_val) == False:
                                    src_val = self.truncate(src_val,int(self.configData["THRESHOLD"]))
                                if type(tgt_val)== numpy.float64 and math.isnan(tgt_val) == False:
                                    tgt_val = self.truncate(tgt_val,int(self.configData["THRESHOLD"]))
                                
                            if src_val != tgt_val and type(src_val)==type(tgt_val):
                                    
                                li = key_val.split('--')
                                for i in range(len(li)):
                                    key = self.keys[i]
                                    self.li2[i].append(li[i])
                                    value = self.li2[i]
                                    comm_dict[key] = value
                                dummy_dict.get('Column_Name').append(field)
                                dummy_dict.get('Source_Value').append(src_val)
                                dummy_dict.get('Target_Value').append(tgt_val)
                                dummy_dict.get('REASON OF FAILURE').append("Difference In Value")                       
                                    
                            elif type(src_val)!= type(tgt_val):
                                        
                                li = key_val.split('--')
                                for i in range(len(li)):
                                    key = self.keys[i]
                                    self.li2[i].append(li[i])
                                    value = self.li2[i]
                                    comm_dict[key] = value
                                dummy_dict.get('Column_Name').append(field)
                                dummy_dict.get('Source_Value').append(src_val)
                                dummy_dict.get('Target_Value').append(tgt_val)
                                dummy_dict.get('REASON OF FAILURE').append("Difference In Datatype")
                                
                comm_dict.update(dummy_dict)
                return comm_dict


    def writeExcel(self,valDict,keyDict):

            logging.info("----------in add excel---------")
            self.logger.info("In Add Excel Function")
            outputFolder = self.data['OUTPUT_FOLDER']
            unique_id = uuid.uuid4()
            excelPath = os.path.join(outputFolder,self.configData['NAME']+str(unique_id)+'.xlsx')
            excel = os.path.join(".","testcases",self.configData['NAME']+str(unique_id)+'.xlsx')
            self.value_df = pd.DataFrame(valDict)
            self.keys_df = pd.DataFrame(keyDict)
            if "AFTER_FILE" in self.configData:
                self.afterMethod()
            if (self.key_check + self.value_check) == 0:
                self.reporter.addRow("Data Validation Report","No MisMatch Value Found", status= status.PASS )
                self.logger.info("----No MisMatch Value Found----")
            else:
                self.logger.info("----Adding Data to Excel----")
                with pd.ExcelWriter(excelPath) as writer1:
                    if self.key_check == 0:
                        pass
                    else:
                        self.keys_df.to_excel(writer1, sheet_name = 'key_difference', index = False)
                    if self.value_check == 0:
                        pass
                    else:
                        self.value_df.to_excel(writer1, sheet_name = 'value_difference', index = False)
                s3_url = None
                try:
                    s3_url = create_s3_link(url=upload_to_s3(DefaultSettings.urls["data"]["bucket-file-upload-api"], bridge_token = self.data["SUITE_VARS"]["bridge_token"], tag="public", username=self.data["SUITE_VARS"]["username"], file=excelPath)[0]["Url"])
                except Exception as e:
                    logging.warn(e)
                if not s3_url:
                    self.reporter.addRow("Data Validation Report","DV Result File: "+'<a href='+excel+'>Result File</a>', status= status.FAIL )
                else:
                    self.reporter.addRow("Data Validation Report","DV Result File: "+'<a href='+s3_url+'>Result File</a>', status= status.FAIL )
            self.reporter.addMisc("common Keys", str(len(self.common_keys)))
            self.reporter.addMisc("Keys Only in Source",str(len(self.keys_only_in_src)))
            self.reporter.addMisc("Keys Only In Target", str(len(self.keys_only_in_tgt)))


    def addReasonOfFailure(self,rof):

        exceptiondata = rof.format_exc().splitlines()
        exceptionarray = [exceptiondata[-1]] + exceptiondata[1:-1]
        self.reporter.addMisc("reason of failure",exceptionarray[0])

    
    def beforeMethod(self):
        """This function
        -checks for the before file tag
        -stores package, module,class and method
        -runs before method if found
        -takes all the data from before method and updates the self object"""

        # check for before_file
        self.logger.info("CHECKING FOR BEFORE FILE___________________________")

        file_str = self.data["config_data"].get("BEFORE_FILE", "")
        if not file_str or file_str == "" or file_str == " ":
            self.logger.info("BEFORE FILE NOT FOUND___________________________")
            self.reporter.addRow("Searching for Before_File steps", "No Before File steps found", status.INFO)

            return
        self.reporter.addRow("Searching for Before_File steps", "Searching for Before File", status.INFO)
        
        file_name = file_str.split("path=")[1].split(",")[0]
        if "CLASS=" in file_str.upper():
            class_name = file_str.split("class=")[1].split(",")[0]
        else:
            class_name = ""
        if "METHOD=" in file_str.upper():
            method_name = file_str.split("method=")[1].split(",")[0]
        else:
            method_name = "before"
        
        self.logger.info("Before file path:- " + file_name)
        self.logger.info("Before file class:- " + class_name)
        self.logger.info("Before file mthod:- " + method_name)
        try:
            file_path = download_common_file(file_name,self.data.get("SUITE_VARS",None))
            file_obj= moduleImports(file_path)
            self.logger.info("Running before method")
            obj_ = file_obj
            before_obj = DvObj(
                pg=self.reporter,
                project=self.project,
                source_df=self.source_df,
                target_df=self.target_df,
                source_columns=self.source_columns,
                target_columns=self.target_columns,
                keys=self.keys,
                env=self.env,
            )
            if class_name != "":
                obj_ = getattr(file_obj, class_name)()
            fin_obj = getattr(obj_, method_name)(before_obj)
            self.extractBeforeObj(fin_obj)
            
        except Exception as e:
            self.logger.info(traceback.print_exc())
            self.reporter.addRow("Executing Before method", f"Some error occurred while searching for before method- {str(e)}", status.ERR)
    
    def afterMethod(self):
        """This function
        -checks for the after file tag
        -stores package, module,class and method
        -runs after method if found
        -takes all the data from after method and updates the self object"""

        self.logger.info("CHECKING FOR AFTER FILE___________________________")

        file_str = self.data["config_data"].get("AFTER_FILE", "")
        if not file_str or file_str == "" or file_str == " ":
            self.logger.info("AFTER FILE NOT FOUND___________________________")
            self.reporter.addRow("Searching for After_File Steps", "No File Path Found", status.INFO)
            return

        self.reporter.addRow("Searching for After_File", "Searching for File", status.INFO)
        
        file_name = file_str.split("path=")[1].split(",")[0]
        if "CLASS=" in file_str.upper():
            class_name = file_str.split("class=")[1].split(",")[0]
        else:
            class_name = ""
        if "METHOD=" in file_str.upper():
            method_name = file_str.split("method=")[1].split(",")[0]
        else:
            method_name = "after"
        self.logger.info("After file path:- " + file_name)
        self.logger.info("After file class:- " + class_name)
        self.logger.info("After file mthod:- " + method_name)
        try:
            file_path = download_common_file(file_name, self.data.get("SUITE_VARS", None))
            file_obj= moduleImports(file_path)
            self.logger.info("Running After method")
            obj_ = file_obj
            after_obj = DvObj(
                pg=self.reporter,
                project=self.project,
                value_df = self.value_df,
                keys_df = self.keys_df,
                env=self.env,
            )
            if class_name != "":
                obj_ = getattr(file_obj, class_name)()
            fin_obj = getattr(obj_, method_name)(after_obj)
            self.extractAfterObj(fin_obj)
        except Exception as e:
            self.reporter.addRow("Executing After method", f"Some error occurred while searching for after method- {str(e)}", status.ERR)

    
    def extractBeforeObj(self, obj):
        """To ofload the data from pyprest obj helper, assign the values back to self object"""

        self.reporter = obj.pg
        self.project = obj.project
        self.source_columns = obj.source_columns
        self.target_columns = obj.target_columns
        self.source_df = obj.source_df
        self.target_df = obj.target_df
        self.keys = obj.keys
        self.env = obj.env
    
    def extractAfterObj(self,obj):

        self.reporter = obj.pg
        self.project = obj.project
        self.value_df = obj.value_df
        self.keys_df = obj.keys_df
        self.env = obj.env





# from libs.Base import Base
from data_compare import data_comp as dc
class Compare():
    def __init__(self, test_id, config_data):
       #testcase specific config can be set here for the object
       #also call the base class __init__ to set base config here
       super(Compare, self).__init__(config_data, test_id)
       self.config_data = config_data
       pass

    def run(self):
       #function for running this testcase
       (dir())
       print(f"Executing testcase {self.testcase_name}")
       res = dc.worker_process((self.test_id,self.config_data))
       r_val, key_only_in_src, key_only_in_tgt,comm_key, diff_count, match_count = res
       print(f"key only in src {key_only_in_src}")
       print(f"key only in tgt {key_only_in_tgt}")
       print(f"commonkey {comm_key}")
       print(f"diff-count {diff_count}")
       print(f"match_count {match_count}")
       pass

if __name__ == "__main__":
    pass



class DvObj:
    def __init__(self, **kwargs):
        """
        source_df and target_df will contains the source and target dataframe
        source_columns and target_columns will contain columns name 
        if we do any change in df columns in before method than we need to do it in source_columns too

        self.keys will always be given in list

        value_df will contain mismatch for values and keys_df will contain the mismatch for keys 
        """
        if "pg" in kwargs:
            self.pg = kwargs['pg']
        if "project" in kwargs:
            self.project = kwargs["project"]
        if "source_df" in kwargs:      
            self.source_df = kwargs["source_df"]
        if "target_df" in kwargs:
            self.target_df = kwargs["target_df"]
        if "source_columns" in kwargs:
            self.source_columns = kwargs["source_columns"]
        if "target_columns" in kwargs:
            self.target_columns = kwargs["target_columns"]
        if "keys" in kwargs:
            self.keys = kwargs["keys"]
        if "env" in kwargs:
            self.env = kwargs["env"]
        if "value_df" in kwargs:
            self.value_df = kwargs["value_df"]
        if "keys_df" in kwargs:
            self.keys_df = kwargs["keys_df"]

    @property
    def returnObj():
        return DvObj()



from gempyp.config import DefaultSettings
import requests
import json
import logging
import traceback
from datetime import datetime
from gempyp.engine import dataUpload


def createJira(jira_body, bridge_token, user_name):
    logging.info("----------- Trying to create Jira Ticket ------------")
    create_jira_api = DefaultSettings.urls["data"]["jira-api"]

    try:
        jira_res = dataUpload._sendData(jira_body, url=create_jira_api, bridge_token=bridge_token, user_name=user_name)
        if jira_res.status_code == 201 or jira_res.status_code == 200:
            jira_json = json.loads(jira_res.text)
            try:
                return jira_json.get("data", None).get("key", None)
            except:
                logging.info("No need to create jira")            
                return None
        else:
            logging.info("Jira API Response - " + str(jira_res.text))
    except Exception as e:
        traceback.format_exc()
        logging.info(e)
        return None


def jiraIntegration(s_run_id, email, access_token, project_id, env, workflow, bridge_token, user_name, suiteName):
    logging.info("---------- In Jira Integration ---------")
    
    jira_id = None
    if workflow:
        workflow = workflow.strip("'").strip('"').split(",")
        workflow = [str(i.strip(" ")) for i in workflow]
   
    jira_body = {
        "email": email, 
        "accessToken": access_token,
        "projectId": project_id, 
        "s_run_id": s_run_id,
        "flow": workflow,
        "env": env,
        "suiteName": suiteName
    }
    if not workflow:
        del jira_body["flow"]
    jira_body = json.loads(json.dumps(str(jira_body).replace("'", '"')))
    try:
        logging.info("----------- Requesting JIRA API ------------")
        jira_id = createJira(jira_body, bridge_token, user_name)
        if jira_id:
            logging.info(f"----------- Jira Id - {jira_id} ------------")
        return jira_id
    except Exception as e:
        traceback.print_exc()
        return None
    



import logging
import inspect
import os
import traceback
import uuid
from typing import Dict, List, Tuple
from gempyp.engine.simpleTestcase import AbstractSimpleTestcase
import getpass
from gempyp.libs.common import download_common_file, moduleImports
from gempyp.libs.gem_s3_common import upload_to_s3, create_s3_link
from gempyp.config import DefaultSettings

def testcaseRunner(testcase_meta: Dict) -> Tuple[List, Dict]:

    """
    actually imports the testcase files and call the run method 
    set the json data that is required to update in db
    """
    logging.info("---------- In testcase Runner -----------")
    config_data: Dict = testcase_meta.get("config_data")
    if testcase_meta.get("default_urls", None):
        DefaultSettings.urls.update(testcase_meta.get("default_urls"))  # only for optimized mode, urls not shared between processes
    logger = config_data.get("LOGGER")
    testcase_meta.pop("config_data")
    try:
        file_name = config_data.get("PATH")
        file_path = download_common_file(file_name,testcase_meta.get("SUITE_VARS",None))
        dynamic_testcase = moduleImports(file_path)
        try:
            # TODO update the config_data to contain some default values
            # GEMPYPFOLDER
            all_classes = inspect.getmembers(dynamic_testcase, inspect.isclass)

            for name, cls in all_classes:
                # currently running only one class easily extensible to run multiple classes
                # from single file
                if (
                    issubclass(cls, AbstractSimpleTestcase)
                    and name != "AbstractSimpleTestcase"
                ):

                    logging.info("------- In subclass check --------")
                    result_data = cls().RUN(cls, config_data, **testcase_meta)


                    break
            # testcase has successfully ran
            # make the output Dict
            output = []

            for data in result_data:
                data["TESTCASEMETADATA"] = testcase_meta
                data["config_data"] = config_data

                singleTestcase = getOutput(data)

                output.append(singleTestcase)
            return output, None

        except Exception as e:
            logger.error("Error occured while running the testcase: {e}".format(e=e))
            return None, getError(e, config_data)
 
    except Exception as e:
        logger.error("Some Error occured while making the testcase: {e}".format(e=e))
        return None, getError(e, config_data)

def getOutput(data):
    tempdict = {}
    # TODO
    # MAKE the TC_RUNID
    unique_id = uuid.uuid4()
    try:
        if os.environ.get('unique_id'):
            unique_id = os.environ.get('unique_id')
    except Exception:
        traceback.print_exc()
    tc_run_id = f"{data['NAME']}_{unique_id}"
    tempdict["tc_run_id"] = tc_run_id
    tempdict["name"] = data["NAME"]
    tempdict["category"] = data["config_data"].get("CATEGORY", None)
    tempdict["status"] = data["STATUS"]
    tempdict["base_user"] = getpass.getuser()
    tempdict["invoke_user"] = data["TESTCASEMETADATA"]["INVOKE_USER"]
    tempdict["machine"] = data["TESTCASEMETADATA"]["MACHINE"]
    tempdict["product_type"] = "GEMPYP"
    tempdict["result_file"] = data["RESULT_FILE"]
    tempdict["start_time"] = data["START_TIME"]
    tempdict["end_time"] = data["END_TIME"]
    tempdict["ignore"] = True if data["TESTCASEMETADATA"].get("IGNORE") else False
    # tempdict["response_time"]="{0:.{1}f} sec(s)".format((data["END_TIME"]-data["START_TIME"]).total_seconds(),2)
    

    all_status = data["json_data"]["meta_data"][2]
    total = 0
    for key in all_status:
        total = total + all_status[key]
    data["json_data"]["meta_data"][2]["TOTAL"] = total

    try:
        log_file= os.path.join('logs',data['NAME']+'_'+unique_id+'.txt')  # ## replacing log with txt for UI compatibility
    except Exception:
        log_file = None
    tempdict["log_file"] = log_file
    try:
        s3_log_file_url= create_s3_link(url=upload_to_s3(DefaultSettings.urls["data"]["bucket-file-upload-api"], bridge_token=data.get("TESTCASEMETADATA").get("SUITE_VARS", None).get("bridge_token",None), username=data.get("TESTCASEMETADATA").get("SUITE_VARS", None).get("username",None), file=data["config_data"].get("log_path", data["config_data"].get("LOG_PATH", "N.A")),tag="public")[0]["Url"])
        s3_log_file_url = f'<a href="{s3_log_file_url}" target=_blank>view</a>'
    except Exception:
         s3_log_file_url = None
    singleTestcase = {}
    singleTestcase["testcase_dict"] = tempdict
    singleTestcase["misc"] = data.get("MISC")
    singleTestcase["json_data"] = data.get("json_data")
    singleTestcase["misc"]["log_file"] = s3_log_file_url
    return singleTestcase

def getError(error, config_data: Dict) -> Dict:
    """
    returning the dictionary of error teatCase data
    """
    error = {}
    error["testcase"] = config_data.get("NAME")
    error["message"] = str(error)
    error["product_type"] = "GEMPYP"
    error["category"] = config_data.get("CATEGORY", None)
    error['log_path'] = config_data.get('log_path', None)
    return error



class RestObj:
    def __init__(self, **kwargs):
        if "pg" in kwargs:
            self.pg = kwargs['pg']
        if "project" in kwargs:
            self.project = kwargs["project"]
        if "request" in kwargs:
            self.request = kwargs["request"]
        if "response" in kwargs:
            self.response = kwargs["response"]
        if "tcname" in kwargs:
            self.tcname = kwargs["tcname"]
        if "variables" in kwargs:
            self.variables = kwargs["variables"]
        if "legacy_res" in kwargs:
            self.legacy_res = kwargs["legacy_res"]
        if "legacy_req" in kwargs:
            self.legacy_req = kwargs["legacy_req"]
        if "request_file" in kwargs:
            self.request_file = kwargs["request_file"]
        if "env" in kwargs:
            self.env = kwargs["env"]

    @property
    def returnObj():
        return RestObj()



import logging
from gempyp.config.xmlConfig import XmlConfig
from gempyp.pyprest.pypRest import PypRest
from gempyp.config.baseConfig import AbstarctBaseConfig
from typing import Type
import getpass
import platform
import argparse
import os
import datetime
from datetime import timezone, datetime


class REngine:
    def __init__(self, **kwargs):
        # parse cli to get restcase name
        parser = argparse.ArgumentParser()
        self.args = self.parseArguments(parser)
        if (self.args.testcase and self.args.config):
            self.path = self.args.config
            self.tcname = self.args.testcase
            self.config = XmlConfig(self.path)
        else:
            self.tcname = kwargs.get("tcname", None)
            self.config = kwargs.get("data", None)
        self.runner()
        pass

    def runner(self):
        config = self.config
        self.setUp(config)
        data = self.formData(config, self.tcname)
        PypRest(data).restEngine()
        logging.info("-----end-------")

    def formData(self, config: Type[AbstarctBaseConfig], tcname):
        data = {}
        # get the testcase list from config and check for the passed testcase
        
        if tcname.upper() in self.CONFIG.getTestcaseConfig().keys():
            self.PARAMS = config.getSuiteConfig()
            data["config_data"] = config.getTestcaseData(tcname)
            data["PROJECT_NAME"] = self.PARAMS["PROJECT"]
            data["ENV"] = self.PARAMS["ENV"]
            # data["S_RUN_ID"] = ""
            data["USER"] = getpass.getuser()
            data["MACHINE"] = platform.node()
            data["OUTPUT_FOLDER"] = ""
        return data

    def setUp(self, config: Type[AbstarctBaseConfig]):
        self.PARAMS = config.getSuiteConfig()
        self.CONFIG = config 
        self.machine = platform.node()
        self.user = getpass.getuser()
        self.current_dir = os.getcwd()
        self.platform = platform.system()
        self.start_time = datetime.now(timezone.utc)
        self.project_name = self.PARAMS["PROJECT"]
        self.report_name = self.PARAMS.get("REPORTNAME")
        self.project_env = self.PARAMS["ENV"]

    def parseArguments(self, parser):
        parser.add_argument("-t", "--testcase",dest="testcase", help="name of the testcase to run")
        parser.add_argument("-config", "--config", help="config file")
        args = parser.parse_args()
        return args


if __name__ == "__main__":
    config = XmlConfig('C:\\Users\\ar.mishra\\gempyp\\tests\\configTest\\legacy_test.xml')
    tcname = "Legacy- Execution-GET"
    REngine(data=config, tcname=tcname)


import os
import sys
import traceback
import common.common_functions as cf
# import validator


class Configurator():
    def __init__(self, args) -> None:
        
        try:
            self.default_json_file = 'def_conf.json'
            if args.config and args.dbfile:
                config_file_path = args.config
                dbfile_file_path = args.dbfile
                if os.path.isfile(config_file_path) and os.path.isfile(dbfile_file_path):
                    config_data = cf.read_json(config_file_path)
                    db_data = cf.read_json(dbfile_file_path)
                    if config_data and db_data:
                        self.config_data = config_data
                        self.db_data = db_data
                    else:
                        self.config_data = None
                        self.db_data = None
                        print("Error reading json file {}, {}".format(config_file_path, dbfile_file_path))
                else:
                    self.config_data = None
                    print("Config file {}, {} does not exists".format(config_file_path, dbfile_file_path))
            else:
                srcqueryfile = args.srcQueryFile
                tgtqueryfile = args.tgtQueryFile
                src_query = args.srcQuery
                tgt_query = args.tgtQuery
                config_issue = False
                if not src_query:
                    if os.path.isfile(srcqueryfile):
                        with open(srcqueryfile) as src:
                            src_query = ' '.join(src.readlines()).strip("\n")
                    else:
                        config_issue = True
                        print("Source query file {} not found".format(srcqueryfile))
                if not tgt_query:
                    if os.path.isfile(tgtqueryfile):
                        with open(tgtqueryfile) as tgt:
                            tgt_query = " ".join(tgt.readlines()).strip("\n")
                    else:
                        config_issue = True
                        print("target query file {} not found".format(tgtqueryfile))
                if not config_issue:
                    self.config_data = {
                        "comparisons":{
                            "1":{
                                "srcDatabase" : args.src,
                                "srcTable" : args.srcTab,
                                "srcSchena" : args.srcSchema,
                                "tgtDatabase" : args.tgtDB,
                                "tgtTable": args.tgtTab,
                                "tgtSchema": args.tgtSchema,
                                "src_query": src_query,
                                "tgt_query": tgt_query,
                                "primary_keys": [x for x in args.primaryKeys.split("--")] 
                            }
                        }
                    }
                else:
                    self.config_data = None
                    print("Source Query file {} or Tgt query file {} not found".format(srcqueryfile, tgtqueryfile))
        except Exception:
            print("Exception occured {}".format(traceback.format_exc()))

    def merge_two_dict(self, x, y):
        z = x.copy()  # starts with s's keys and values
        z.update(y)   # modifies z with y's keys and values
        return z

    def set_config(self):
        """
        Combining and seggrating config provided by user and application default config in this method
        """
        try:
            r_val = 0
            if self.config_data:
                config = self.config_data.copy()
                print("Before setting config {}".format(config))
                def_conf = cf.read_json(self.default_json_file)
                print("def_conf is {}".format(def_conf))
                if def_conf:
                    for comp_id in config["comparisons"]:
                        config["comparisons"][comp_id] = self.merge_two_dict(def_conf,config["comparisons"][comp_id])
                else:
                    print("Default conf not found")
                self.config_data = config
                print("After setting config {}".format(config))
                print("Config set successfully")
            else:
                r_val = 1

        except Exception:
            print("Exception occured in set_config {}".format(traceback.format_exc()))
            r_val = 1
        return r_val
    
    def get_config(self):
        return self.config_data, self.db_data



import sys
import os
import json
import traceback
import pandas as pd
import time
from cryptography.fernet import Fernet
import warnings
from getpass import getpass, getuser

def read_json(file_path):
    try:
        #print(file_path)
        with open(file_path) as fobj:
            res = json.load(fobj)
    except Exception:
        res = None
        print("Exception in read_json for file path {} is {}".format(file_path, traceback.format_exc()))
    return res

def write_json(data, file_path):
    try:
        r_val = 1
        with open(file_path,'w') as fobj:
            json.dump(data, fobj, indent=4, sort_keys=True)
        r_val = 0
    except Exception :
        print("Exception in write json for file_path {} is {} ".format(file_path, traceback.format_exc()))
    return r_val

def mem_usage(pandas_obj):
    if isinstance(pandas_obj, pd.DataFrame):
        usage_b = pandas_obj.memory_usage(deep=True).sum()
    else:  # we assume if it is not a df it is a series
        usage_b = pandas_obj.memory_usage(deep= True)
    usage_mb = usage_b / 1024 ** 2  #converting bytes to megabytes
    return "{:03.2f}".format(usage_mb)

def timeit(method):
    def timed(*args, **kw):
        ts = time.time()
        result = method(*args, **kw)
        te = time.time()
        print("{} {} seconds".format(method.__name__, (te-ts)))
        return result
    return timed

def exc_handler(method):
    def check_exc(*args, **kw):
        try:
            res = method(*args, **kw)
        except Exception:
            res = None
            print("Exception occured in {} as {}".format(method.__name__, traceback.format_exc()))
        return res
    return check_exc

def encrypt(passwd, key=None):
    try:
        if not key:
            cur_user = getuser()
            KEY_DIR = '/home/'+cur_user+'/.ssh'
            key_file_name = "." + cur_user + ".key"
            key_file_path = os.path.join(os.path.abspath(KEY_DIR), key_file_name)
            print("------Keyfile path {}------".format(key_file_path))
            with open(key_file_path, "w") as f:
                key = Fernet.generate_key()
                f.write(key.decode('UTF-8'))
                pass
            print("Changing file permission to user read-only")
            os.chmod(key_file_path, 0o600)
        f = Fernet(key)
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            token = f.encrypt(passwd.encode("utf-8"))
    except Exception:
        traceback.print_exc()
        token = None
        pass
    return token

def decrypt(enc_pass):
    cur_user = getuser()
    KEY_DIR = KEY_DIR = '/home/'+cur_user+'/.ssh'
    key_file_name = "." + cur_user + ".key"
    key_file_path = os.path.join(os.path.abspath(KEY_DIR), key_file_name)
    print("Keyfile path is {}".format(key_file_path))
    if not os.path.exists(key_file_path):
        print(
            "Keyfile does not exist. Exiting"
        )
        sys.exit(1)
    else:
        print("Keyfile exists. Retrieving key")
        with open(key_file_path) as f:
            key = f.readline()
    try:
        f = Fernet(key)
        pass
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            enc_pass = enc_pass.encode('utf-8')
            dec_passwd = f.decrypt(enc_pass)
            dec_passwd = dec_passwd.decode('utf-8')
    except Exception:
        dec_passwd = None
        traceback.print_exc()
    return dec_passwd
    
if __name__ == '__main__':
    pass








from datetime import datetime, date, timedelta
from sqlite3 import TimeFromTicks
import time
import traceback
import pytz
import random
import uuid


#TODO: functions we can add in the predefined functions 
"""
getDate according to time zone.
getTime according to time zone.
getUniqueNo in the given range
randomNo number
random out of a list
generate list of values(start value, end val, iterator val)
getdatetime(format, offset, timezone)
transform_date(date,offset,timezone)

"""



class PredefinedFunctions:
    def __init__(self, pyprest_obj):
        self.pyprest_obj = pyprest_obj
        # logger.info("________________________________ Inside Predefined Functions _________________________________________")
        self.date_formats = {
            'ddmmyyyy': '%d/%m/%Y',
            'ddmonyyyy': "%d %b, %Y",
            'monddyyyy': '%b %d, %Y',
            'ddmmyy': '%d%m%y',
        }  # add mode datetime formats


    def parseParams(self, param_str):
        """parsing string parameters ( all the params are strings )"""
        if '[' in param_str:
            br_start = param_str.find('[')
            br_end = param_str.find(']')
            list_ = param_str[br_start + 1:br_end].split(",")
            params_list = param_str[:br_start].split(",") + [list_] + param_str[br_end + 1:].split(",")
            params_list = [i for i in params_list if i != '' and i != " "]

        else:
            params_list = param_str.split(",")
        if len(params_list) > 0:
            params_list = [each.strip(" ") if isinstance(each, str) else each for each in params_list]
        return params_list

    def rand(self, start, end):
        """get random number out of a given range"""
        try:
            if int(start) < int(end):
                self.rand_number = random.randint(int(start), int(end))
                return self.rand_number
            else:
                self.pyprest_obj.logger.info("Start and end values are not defined properly")
        except Exception as e:
            self.pyprest_obj.logger.error(str(e)) 

    # return uuid
    def uuid(self, *kwargs):
        return str(uuid.uuid4())    

    def epoch(self, str):
        """
        Return epoch time in seconds
        if "milli" is passed as a parameter then returns time in epoch milli"""
        millis = False
        type = self.parseParams(str)
        if len(type) > 0:
            if "mil" in type[0].lower():
                millis = True
        try:
            if millis == True:
                return round(time.time()*1000, 0)
            else:
                return round(time.time(), 0)
        except Exception as e:
            self.pyprest_obj.logger.info(str(e))

    def unique(self, len_):
        """
        Return unique number of the length given"""
        len_ = int(len_.strip('"').strip("'"))
        number_ = str(str(time.time()*1000).split('.')[0])
        number_ = self.numberGenerator(number_, len_)
        start = len(str(number_)) - len_
        number_ = int(number_[start:])
        return number_

    def numberGenerator(self, num, len_):
        """generate a number greater than the length given"""
        if len_ > len(num):
            num = num + num
            if len_ > len(num):
                return self.numberGenerator(num, len_)
            else:
                return num
        else:
            return num


    #get random element from a list
    def getRandomFromAlist(self, listName):
        """
        acceps a list of elements.
        Returns a random element from the lists
        """
        try:
            startIdxList = 0 
            endIdxList = len(listName)-1
            randIndex = random.randint(startIdxList, endIdxList)
            return listName[randIndex]
        except Exception as e:
            self.pyprest_obj.logger.info(str(e))   


    #get date from now function it will add a value which is provided as a parameter to the current date and will return it to the use.
    def getDateFromNow(self, *args):
        """
        get date from now function it will add a value which 
        is provided as a parameter to the current date and will return it to the use.

        params_str should contain comma separated values-
        days,
        date format """
        n_value = int(args[0].strip('"').strip("'"))
        try:
            date_format = args[1].strip('"').strip("'")
        except Exception as e:
            self.pyprest_obj.logger.info(traceback.print_exc())
            date_format = 'ddmmyyyy'
        try:
            self.data_N_days_after = date.today() + timedelta(days = int(n_value))
            self.value = self.data_N_days_after.strftime(str(self.date_formats[date_format]))
            return self.value
        except Exception as e:
            self.pyprest_obj.logger.info(traceback.print_exc())
            self.pyprest_obj.logger.info("Error occured while excuting getDateFromNow() function")

    # getDate function with format
    def curr_timestamp(self, dateformat="", tz="UTC"):
        """
        return current datetime in required format
        if user gives a timezone, he/she will get time in that timezone
        
        currently supported date formats are-
        ddmmyy
        ddmonyyyy
        ddmmyyyy
        monddyyyy
        .
        """
        current_timestamp = datetime.now().strftime("%d%m%y")
        try: 
            self.pyprest_obj.logger.info("inside get date ")
            if not tz:
                date_time = datetime.now()
            else:
                date_time  = datetime.now(pytz.timezone(tz))
            try:
                current_timestamp = date_time.strftime(self.date_formats.get(dateformat.strip('"').strip("'").lower(), '%d%m%y'))
            except Exception as e:
                self.pyprest_obj.logger.info(str(e))
        except Exception as e:
            self.pyprest_obj.logger.info(str(e))
        return current_timestamp
        

    # TODO
    #get time function with timezone, time_format, twenty_four_hour_format
    def getTime(self,tz = "", time_format = "", twenty_four_hour_Format = True):
        try:
            self.pyprest_obj.logger.info("Execution of getTime")
            try:
                # print(tz)
                if not tz: 
                    date_time = datetime.datetime.now()  
                else:
                    try:
                        date_time = datetime.datetime.now(pytz.timezone(tz))
                    except Exception as e:
                         self.pyprest_obj.logger.info(str(e))
            except Exception as e:
                self.pyprest_obj.logger.info(str(e))
            if not time_format and  twenty_four_hour_Format is False  :
                self.current_time = date_time.strftime("%I:%M:%S")
            elif not time_format and twenty_four_hour_Format is True:
                self.current_time = current_time = date_time.strftime("%H:%M:%S")
            elif time_format == "hh:mm" and twenty_four_hour_Format is False :
                 self.current_time = date_time.strftime("%I:%M")
                 if int(date_time.strftime("%H"))>=12:
                    return self.current_time+" PM"
                 else:
                    return self.current_time+" AM"
            elif time_format == "hh:mm" and twenty_four_hour_Format is True :
                self.current_time = date_time.strftime("%H:%M")
            elif twenty_four_hour_Format is False:
                
                self.current_time = date_time.strftime("%I:%M:%S")
                if(int(date_time.strftime("%H"))>=12):
                    return self.current_time+" PM"
                else:
                    return self.current_time+" AM"
            else:
                self.current_time = date_time.strftime("%H:%M:%S")
            return self.current_time
        except Exception as e:
            self.pyprest_obj.logger.info(str(e))

    def generateListofValues(self, start_val, end_val, iter_val):
        pass

    def alpha(self, *args):
        pass
        


from datetime import datetime, timezone
from typing import Dict
from gempyp.libs.enums.status import status
from gempyp.reporter.reportGenerator import TemplateData


class TestcaseReporter:
    """the class need to be extended by all the testcases"""

    def __init__(self, project_name: str = None, testcase_name: str = None):
        """initializes the reportfile to start making the report
        :param project_name: The name of the project. Will be overidden by the name
                            present in the config if the config is given.
        :param testcase_name: testcase name. Will be overidden by the name present in the
                            config if the config is present.
        :param destructor: to call the destructor function if True. if False user have to manually call the function.
        :param extraColumns: any extra column required in the result file
        """
        self.project_name = project_name.strip()
        self.testcase_name = testcase_name.strip()
        self.begin_time = datetime.now(timezone.utc)
        self.end_time = None
        self._misc_data = {}
        self.json_data = None
        self._is_destructor_called = False
        self.status_count = {k: 0 for k in status}
        self.template_data = TemplateData()
        self.logger = None

        # can be overiiden for custom result file
        self.result_file_name = None
        # can be overiiden or will be automatically decided in the end
        self.status = None

        # starting the new Report
        self.template_data.newReport(self.project_name, self.testcase_name)

    def addMisc(self, key: str, value):
        """
        add the misc data to the report by calling it as reporter.addMisc()
        takes two arguments one is column name and other one is value
        """
        if key in self._misc_data:
            if value in self._misc_data[key]:
                pass
            else:
                self._misc_data[key] = self._misc_data[key]+ "," +'<br>' + value
        else:
            self._misc_data[key] = value

    def getMisc(self, key: str):
        """
        returns the misc data for the sprcified key
        returns none if no data is found
        """
        return self._misc_data.get(key, None)

    def addRow(
        self,
        test_step: str,
        description: str,
        status: status,
        file: str = None,
        link_name: str = "Click Here",
        **kwargs,
    ):
        """
        add the new row to the report 
        takes three argument first is testname,second is description of test and third is status 
        calls the newRow method of reportGenerator 
        
        """
        self.status_count[status] += 1

        attachment = None
        if file:
            # link = self.addLink(file, linkName)
            attachment = {"URL": file, "linkName": link_name}
        # add the new row
        self.template_data.newRow(
            test_step, description, status.name, attachment=attachment, **kwargs
        )

    def addLink(self, file: str, linkName: str):
        """
        return the anchor tab instead of the link
        """
        # if you want to add the file to s3 we can do it here

        return f""" <a href='{file}'>{linkName}</a> """

    def finalizeReport(self):
        """
        the destructor after the call addRow will not work and will call finalizeResult method of reportGenerator file
        """
        # only call the destructor once
        if self._is_destructor_called:
            return
        self._is_destructor_called = True

        if not self.status:
            self.status = self.findStatus()
            print(self.findStatus())
        self.end_time = datetime.now(timezone.utc)
        for key in list(self.status_count):
            if self.status_count[key] == 0:
                del self.status_count[key]
        self.template_data.finalizeResult(self.begin_time, self.end_time, self.status_count)

    def findStatus(self):
        """
        method will return whether the status is pass or fail or warn
        """
        for i in status:
            if self.status_count[i] > 0:
                return i.name
        return status.INFO.name
            

    def serialize(self) -> Dict:
        """
        used to assign some values to the variables that are further used in json that is uploaded to db
        takes values from reporter function and other variables in the same file
        """
        result_data = {}
        result_data["NAME"] = self.testcase_name
        result_data["PROJECT_NAME"] = self.project_name
        result_data["STATUS"] = self.status
        result_data["RESULT_FILE"] = self.result_file_name
        result_data["STEPS_COUNT"] = self.status_count
        result_data["MISC"] = self._misc_data
        result_data["START_TIME"] = self.begin_time
        result_data["END_TIME"] = self.end_time
        result_data["json_data"] = self.json_data
        return result_data



from abc import ABC, abstractmethod
import traceback
from typing import Dict
import logging
import os


class AbstarctBaseConfig(ABC):
    def __init__(self, *args, **kwargs):
        self._CONFIG = {}
        self.cli_config ={}
        self.total_yflag_testcase =0
        
        try:
            self.parse(*args, **kwargs)
            # filter removed from here because we need to apply filter after updating data with cli input(if given)
            # self.update()
            logging.info("----------- Xml parsing completed ------------")
        except Exception as e:
            traceback.print_exc()
            logging.error("failed to parse the config: {e}".format(e=e))

    def getSuiteConfig(self) -> Dict:
        # logging.info("^^^^^^^^^^^^^ \n {suite_data} \n^^^^^^^^^".format(suite_data=self._CONFIG["SUITE_DATA"]))
        self.filter()
        return self._CONFIG["SUITE_DATA"]

    def getTestcaseConfig(self) -> Dict:
        """reutrn the testCaseData to filter method"""
        # logging.info("--------testCaseDict--------\n {testcaseDict} \n----------".format(testcaseDict=self._CONFIG["TESTCASE_DATA"]))
        return self._CONFIG["TESTCASE_DATA"]
    
    def getSubtestcasesConfig(self) -> Dict:
        return self._CONFIG["SUBTESTCASES_DATA"]

    def getTestcaseData(self, testcaseName: str) -> Dict:
        return self._CONFIG["TESTCASE_DATA"].get(testcaseName.upper(), None)  ## uppercase
    
    def getSubTestcaseData(self, testcaseName: str) -> Dict:
        return self._CONFIG["SUBTESTCASES_DATA"].get(testcaseName.upper(), None)  ## uppercase

    def getTestcaseLength(self) -> int:
        """
        return the length of testCase
        """
        return len(self._CONFIG.get("TESTCASE_DATA", []))

    @abstractmethod
    def parse(self, *args, **kwargs):
        """
        override this method in xmlConfig
        """
        pass

    def filter(self):
        """
        filter the testcases that need to be ignored based on the run value and category sets
        """
        testcase_data = self.getTestcaseConfig()
        filtered_dict = {}
        filtered_dict_subtestcases = {}
        ###code for passing testcases through cli jira-113
        if "TESTCASE_LIST" in self._CONFIG['SUITE_DATA']:
            test = {}
            testcase = self._CONFIG['SUITE_DATA']["TESTCASE_LIST"]
            if testcase[0] == '[':
                testcase = testcase[1:-1]
                testcase = [i.strip(" ").upper() for i in testcase.split(",")]  ## uppercase
            else:
                testcase = [i.strip(" ").upper() for i in testcase.split(",")]  ## uppercase
            for key, value in testcase_data.items():
                if key in testcase:
                    test[key] = value
            testcase_data = test

        for key, value in testcase_data.items():
            testcases = ""
            # if(value.get("RUN_FLAG", "N").upper()=="Y" and "SUBTESTCASES" in value.keys()):
            #     testcases=value.get("SUBTESTCASES").upper().split(",")  ## uppercase
            #     testcases.append(key)
            
            # if value.get("RUN_FLAG", "N").upper() != "Y":  # to be removed
            #     continue

            # if self.filter_category(value):
            #     continue

            # filtered_dict[key] = value
            if value.get("RUN_FLAG", "Y").upper() == "Y":
                if self.filter_category(value):
                    continue
                self.total_yflag_testcase += 1
                filtered_dict[key] = value
                if "SUBTESTCASES" in value.keys():
                    testcases=value.get("SUBTESTCASES").upper().split(",")
                    testcases.append(key)
            if(len(testcases)>0):
                for i in range(len(testcases)):
                    if(testcases[i] in testcase_data.keys()):
                        filtered_dict_subtestcases[testcases[i]]=testcase_data.get(testcases[i])
        self._CONFIG["SUBTESTCASES_DATA"] = filtered_dict_subtestcases
        self._CONFIG["TESTCASE_DATA"] = filtered_dict

    def filter_category(self, value):
        """ filtering the categories that we want to run, if any. Returns 0 if we want to run the testcase"""
        exp_category = None
        this_category = []
        if self._CONFIG['SUITE_DATA'].get("CATEGORY", None):
            exp_category = self._CONFIG['SUITE_DATA']["CATEGORY"] if isinstance(self._CONFIG['SUITE_DATA']["CATEGORY"], list) else str(self._CONFIG['SUITE_DATA']["CATEGORY"]).upper().split(",")
        if value.get("CATEGORY", None):
            this_category = value.get("CATEGORY") if isinstance(value.get("CATEGORY"), list) else str(value.get("CATEGORY")).upper().split(",")
        if exp_category:
            if not len(list(set([i.lower() for i in exp_category]) & set([i.lower() for i in this_category]))) > 0:
                return 1
        return 0

    # TODO
    def update(self):
        """to update the data that is passed by cli"""
        try:
            for key in self._CONFIG['SUITE_DATA'].get('ENV-VARS', {}):
                os.environ[key.lower()] = self._CONFIG['SUITE_DATA'].get('ENV-VARS', {}).get(key, None)
        except Exception as error:
            logging.error("Error in updating environment variable - " + str(error))
        try:
            for key in self._CONFIG['SUITE_DATA'].keys():
                value=self._CONFIG['SUITE_DATA'][key]
                if value and ("$[#ENV." in value):
                        envValue=value
                        value=value.replace("$[#ENV.","").strip("]").lower()
                        self._CONFIG['SUITE_DATA'][key]=os.environ.get(value)
                        self._CONFIG['SUITE_DATA']["SUITE_VARS"][envValue.strip("$[#").strip("]").replace(".","_").upper()]=os.environ.get(value)
                if value and ("$[#" in value):
                        value=value.strip("$[#").strip("]").lower()
                        self._CONFIG['SUITE_DATA'][key]=os.environ.get(value)
        except Exception as error:
            traceback.print_exc()
            logging.error("error occurs in finding environment variable - " + str(error))
        try:
            for element in self.cli_config.keys():
                if self.cli_config[element]:
                    if str(element) in self._CONFIG['SUITE_DATA']:
                        if(element=="ENV" and len(self.cli_config[element].split(","))>0):
                            self._CONFIG['SUITE_DATA']["ENV"]=self.cli_config[element].split(",")[0]
                            self._CONFIG['SUITE_DATA'][self.cli_config[element].split(",")[0].upper()]=self.cli_config[element].split(",")[1]
                        else:
                            self._CONFIG['SUITE_DATA'][element] = self.cli_config[element]
                    else:
                        self._CONFIG['SUITE_DATA'][element] = self.cli_config[element]
        except Exception as error:
            logging.error("error occurs in update" + str(error))
        
        



import os
import sys
import datetime
import time

class Comparator():

    def __init__(self, config, src_data, tgt_data, src_header, tgt_header) -> None:
        if config["comp_type"].lower() =="python":
            import data_compare.core.python_obj_comparator as poc
            self.comp_obj = poc.ObjComparator(src_data, tgt_data, config["primary_keys"], config["tolerance"], src_header, tgt_header )
        elif config["comp_type"].lower() =='pandas':
            print("not impemented yet")
    
    def compare(self):
        return self.comp_obj.startCompare()
